from __future__ import annotations

from datetime import datetime
from typing import Optional, TypeVar, Type, Generic

import orjson

from buz.event import Event
from buz.kafka.infrastructure.cdc.cdc_message import CDCPayload
from buz.kafka.infrastructure.deserializers.bytes_to_message_deserializer import BytesToMessageDeserializer
from buz.kafka.infrastructure.deserializers.implementations.cdc.cannot_restore_event_from_cdc_payload_exception import (
    CannotRestoreEventFromCDCPayloadException,
)
from buz.kafka.infrastructure.deserializers.implementations.cdc.cdc_record_bytes_to_cdc_payload_deserializer import (
    CDCRecordBytesToCDCPayloadDeserializer,
)

T = TypeVar("T", bound=Event)


class CDCRecordBytesToEventDeserializer(BytesToMessageDeserializer[Event], Generic[T]):
    __STRING_ENCODING = "utf-8"

    def __init__(self, event_class: Type[T]) -> None:
        self.__event_class = event_class
        self.__cdc_record_bytes_to_cdc_payload_deserializer = CDCRecordBytesToCDCPayloadDeserializer()

    def deserialize(self, data: bytes) -> T:
        cdc_payload = self.__cdc_record_bytes_to_cdc_payload_deserializer.deserialize(data)
        try:
            payload_dict = orjson.loads(cdc_payload.payload)
            return self.__event_class.restore(
                id=cdc_payload.event_id,
                created_at=self.__get_created_at_in_event_format(cdc_payload.created_at),
                metadata=self.__deserialize_metadata(cdc_payload.metadata),
                **payload_dict,
            )
        except Exception as exception:
            raise CannotRestoreEventFromCDCPayloadException(cdc_payload, exception) from exception

    def __get_created_at_in_event_format(self, cdc_payload_created_at: str) -> str:
        created_at_datetime = datetime.strptime(cdc_payload_created_at, CDCPayload.DATE_TIME_FORMAT)
        return created_at_datetime.strftime(Event.DATE_TIME_FORMAT)

    def __deserialize_metadata(self, metadata: Optional[str]) -> dict:
        if metadata is None:
            return {}
        return orjson.loads(metadata)



from __future__ import annotations


class ConsumerGroupNotFoundException(Exception):
    def __init__(self, consumer_group: str) -> None:
        super().__init__(f'The consumer group "{consumer_group}" has not been found')



from __future__ import annotations

import orjson
from cachetools import LRUCache
from dacite import from_dict

from buz.kafka.infrastructure.cdc.cdc_message import CDCPayload
from buz.kafka.infrastructure.deserializers.implementations.cdc.cannot_decode_cdc_message_exception import (
    CannotDecodeCDCMessageException,
)


class CDCRecordBytesToCDCPayloadDeserializer:
    __STRING_ENCODING = "utf-8"
    __cached_deserialization: LRUCache[str, CDCPayload] = LRUCache(maxsize=128)

    def deserialize(self, data: bytes) -> CDCPayload:
        decoded_string = data.decode(self.__STRING_ENCODING)
        if decoded_string not in CDCRecordBytesToCDCPayloadDeserializer.__cached_deserialization:
            try:
                CDCRecordBytesToCDCPayloadDeserializer.__cached_deserialization[
                    decoded_string
                ] = self.__get_cdc_payload_from_string(decoded_string)
            except Exception as exception:
                raise CannotDecodeCDCMessageException(decoded_string, exception) from exception

        return CDCRecordBytesToCDCPayloadDeserializer.__cached_deserialization[decoded_string]

    def __get_cdc_payload_from_string(self, decoded_string: str) -> CDCPayload:
        decoded_record: dict = orjson.loads(decoded_string)
        payload = decoded_record.get("payload")
        if not isinstance(payload, dict):
            raise ValueError("The provided payload value is not valid")

        return from_dict(CDCPayload, payload)



from buz.kafka.domain.exceptions.not_valid_kafka_message_exception import NotValidKafkaMessageException
from buz.kafka.infrastructure.cdc.cdc_payload import CDCPayload


class CannotRestoreEventFromCDCPayloadException(NotValidKafkaMessageException):
    def __init__(self, cdc_payload: CDCPayload, exception: Exception) -> None:
        self.exception = exception
        self.cdc_payload = cdc_payload
        super().__init__(
            f'Couldn\'t build a message from CDC payload "{self.cdc_payload}". Raised from "{str(exception)}"'
        )



from dataclasses import dataclass
from typing import Optional

from buz.event.infrastructure.models.process_context import ProcessContext


@dataclass(frozen=True)
class CDCProcessContext(ProcessContext):
    captured_at_ms: Optional[int] = None



from dataclasses import dataclass

from buz import Message


@dataclass(frozen=True)
class Event(Message):
    @classmethod
    def fqn(cls) -> str:
        return f"event.{super().fqn()}"

    def partition_key(self) -> str:
        return self.fqn()
